@@ -73,8 +73,10 @@ proto_crypt_secret(const char * filename)
 	if ((K = malloc(sizeof(struct proto_secret))) == NULL)
 		goto err0;
 
-	/* Open the file. */
-	if ((f = fopen(filename, "r")) == NULL) {
+	/* Open the file, or use stdin if requested. */
+	if (strcmp(filename, STDIN_FILENAME) == 0) {
+		f = stdin;
+	} else if ((f = fopen(filename, "r")) == NULL) {
 		warnp("Cannot open file: %s", filename);
 		goto err1;
 	}
@@ -88,12 +90,18 @@ proto_crypt_secret(const char * filename)
 
 	/* Did we hit EOF? */
 	if (!feof(f)) {
-		warnp("Error reading file: %s", filename);
+		if (f == stdin) {
+			warnp("Error reading from stdin");
+		} else {
+			warnp("Error reading file: %s", filename);
+		}
+
 		goto err2;
 	}
 
-	/* Close the file. */
-	fclose(f);
+	/* Close the file if it isn't stdin. */
+	if (f != stdin)
+		fclose(f);
 
 	/* Compute the final hash. */
 	SHA256_Final(K->K, &ctx);
@@ -102,7 +110,8 @@ proto_crypt_secret(const char * filename)
 	return (K);
 
 err2:
-	fclose(f);
+	if (f != stdin)
+		fclose(f);
 err1:
 	free(K);
 err0:
@@ -150,7 +159,7 @@ proto_crypt_dhmac(const struct proto_secret * K,
 
 /**
  * is_not_one(x, len):
- * Returns non-zero if the big-endian value stored at (${x}, ${len}) is not
+ * Return non-zero if the big-endian value stored at (${x}, ${len}) is not
  * equal to 1.
  */
 static int
@@ -167,15 +176,15 @@ is_not_one(const uint8_t * x, size_t len)
 }
 
 /**
- * proto_crypt_dh_validate(yh_r, dhmac_r, requirefps):
+ * proto_crypt_dh_validate(yh_r, dhmac_r, requirepfs):
  * Return non-zero if the value ${yh_r} received from the remote party is not
  * correctly MACed using the diffie-hellman parameter MAC key ${dhmac_r}, or
  * if the included y value is >= the diffie-hellman group modulus, or if
- * ${requirefps} is non-zero and the included y value is 1.
+ * ${requirepfs} is non-zero and the included y value is 1.
  */
 int
 proto_crypt_dh_validate(const uint8_t yh_r[PCRYPT_YH_LEN],
-    const uint8_t dhmac_r[PCRYPT_DHMAC_LEN], int requirefps)
+    const uint8_t dhmac_r[PCRYPT_DHMAC_LEN], int requirepfs)
 {
 	uint8_t hbuf[32];
 
@@ -192,7 +201,7 @@ proto_crypt_dh_validate(const uint8_t yh_r[PCRYPT_YH_LEN],
 		return (1);
 
 	/* If necessary, enforce that the diffie-hellman value is != 1. */
-	if (requirefps) {
+	if (requirepfs) {
 		if (! is_not_one(&yh_r[0], CRYPTO_DH_PUBLEN))
 			return (1);
 	}
@@ -202,18 +211,18 @@ proto_crypt_dh_validate(const uint8_t yh_r[PCRYPT_YH_LEN],
 }
 
 /**
- * proto_crypt_dh_generate(yh_l, x, dhmac_l, nofps):
+ * proto_crypt_dh_generate(yh_l, x, dhmac_l, nopfs):
  * Using the MAC key ${dhmac_l}, generate the MACed diffie-hellman handshake
  * parameter ${yh_l}.  Store the diffie-hellman private value in ${x}.  If
- * ${nofps} is non-zero, skip diffie-hellman generation and use y = 1.
+ * ${nopfs} is non-zero, skip diffie-hellman generation and use y = 1.
  */
 int
 proto_crypt_dh_generate(uint8_t yh_l[PCRYPT_YH_LEN], uint8_t x[PCRYPT_X_LEN],
-    const uint8_t dhmac_l[PCRYPT_DHMAC_LEN], int nofps)
+    const uint8_t dhmac_l[PCRYPT_DHMAC_LEN], int nopfs)
 {
 
 	/* Are we skipping the diffie-hellman generation? */
-	if (nofps) {
+	if (nopfs) {
 		/* Set y_l to a big-endian 1. */
 		memset(yh_l, 0, CRYPTO_DH_PUBLEN - 1);
 		yh_l[CRYPTO_DH_PUBLEN - 1] = 1;
@@ -236,11 +245,11 @@ err0:
 }
 
 /**
- * proto_crypt_mkkeys(K, nonce_l, nonce_r, yh_r, x, nofps, decr, eh_c, eh_s):
+ * proto_crypt_mkkeys(K, nonce_l, nonce_r, yh_r, x, nopfs, decr, eh_c, eh_s):
  * Using the protocol secret ${K}, the local and remote nonces ${nonce_l} and
  * ${nonce_r}, the remote MACed diffie-hellman handshake parameter ${yh_r},
  * and the local diffie-hellman secret ${x}, generate the keys ${eh_c} and
- * ${eh_s}.  If ${nofps} is non-zero, we are performing weak handshaking and
+ * ${eh_s}.  If ${nopfs} is non-zero, we are performing weak handshaking and
  * y_SC is set to 1 rather than being computed.  If ${decr} is non-zero,
  * "local" == "S" and "remote" == "C"; otherwise the assignments are opposite.
  */
@@ -249,7 +258,7 @@ proto_crypt_mkkeys(const struct proto_secret * K,
     const uint8_t nonce_l[PCRYPT_NONCE_LEN],
     const uint8_t nonce_r[PCRYPT_NONCE_LEN],
     const uint8_t yh_r[PCRYPT_YH_LEN], const uint8_t x[PCRYPT_X_LEN],
-    int nofps, int decr,
+    int nopfs, int decr,
     struct proto_keys ** eh_c, struct proto_keys ** eh_s)
 {
 	uint8_t nonce_y[PCRYPT_NONCE_LEN * 2 + CRYPTO_DH_KEYLEN];
@@ -263,7 +272,7 @@ proto_crypt_mkkeys(const struct proto_secret * K,
 	memcpy(&nonce_y[PCRYPT_NONCE_LEN], nonce_s, PCRYPT_NONCE_LEN);
 
 	/* Are we bypassing the diffie-hellman computation? */
-	if (nofps) {
+	if (nopfs) {
 		/* We sent y_l = 1, so y_SC is also 1. */
 		memset(&nonce_y[PCRYPT_NONCE_LEN * 2], 0,
 		    CRYPTO_DH_KEYLEN - 1);
@@ -317,7 +326,7 @@ proto_crypt_enc(uint8_t * ibuf, size_t len, uint8_t obuf[PCRYPT_ESZ],
 	memset(&obuf[len], 0, PCRYPT_MAXDSZ - len);
 
 	/* Add the length. */
-	be32enc(&obuf[PCRYPT_MAXDSZ], len);
+	be32enc(&obuf[PCRYPT_MAXDSZ], (uint32_t)len);
 
 	/* Encrypt the buffer in-place. */
 	crypto_aesctr_buf(k->k_aes, k->pnum, obuf, obuf, PCRYPT_MAXDSZ + 4);
@@ -373,7 +382,7 @@ ssize_t proto_crypt_dec(uint8_t ibuf[PCRYPT_ESZ], uint8_t * obuf,
 	memcpy(obuf, ibuf, len);
 
 	/* Return the decrypted length. */
-	return (len);
+	return ((ssize_t)len);
 }
 
 /**
