@@ -1,14 +1,13 @@
 #include <sys/socket.h>
 
-#include <inttypes.h>
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <unistd.h>
 
-#include "asprintf.h"
 #include "events.h"
-#include "sha256.h"
+#include "getopt.h"
+#include "parsenum.h"
 #include "sock.h"
 #include "warnp.h"
 
@@ -20,28 +19,27 @@
 static int
 callback_conndied(void * cookie)
 {
+	int * conndone = cookie;
 
-	(void)cookie; /* UNUSED */
+	/* Quit event loop. */
+	*conndone = 1;
 
-	/* We're done! */
-	exit(0);
+	/* Success! */
+	return (0);
 }
 
 static void
 usage(void)
 {
 
-	fprintf(stderr, "usage: spipe -t <target socket> -k <key file>"
-	    " [-f | -g] [-j] [-o <connection timeout>]\n");
+	fprintf(stderr,
+	    "usage: spipe -t <target socket> -k <key file>"
+	    " [-f | -g] [-j]\n"
+	    "    [-o <connection timeout>]\n"
+	    "       spipe -v\n");
 	exit(1);
 }
 
-/* Simplify error-handling in command-line parse loop. */
-#define OPT_EPARSE(opt, arg) do {					\
-	warnp("Error parsing argument: -%c %s", opt, arg);		\
-	exit(1);							\
-} while (0)
-
 int
 main(int argc, char * argv[])
 {
@@ -50,61 +48,75 @@ main(int argc, char * argv[])
 	int opt_g = 0;
 	int opt_j = 0;
 	const char * opt_k = NULL;
+	int opt_o_set = 0;
 	double opt_o = 0.0;
 	const char * opt_t = NULL;
 
 	/* Working variables. */
 	struct sock_addr ** sas_t;
 	struct proto_secret * K;
-	int ch;
+	const char * ch;
 	int s[2];
+	int conndone = 0;
+	void * conn_cookie;
 
 	WARNP_INIT;
 
 	/* Parse the command line. */
-	while ((ch = getopt(argc, argv, "fgjk:o:t:")) != -1) {
-		switch (ch) {
-		case 'f':
+	while ((ch = GETOPT(argc, argv)) != NULL) {
+		GETOPT_SWITCH(ch) {
+		GETOPT_OPT("-f"):
 			if (opt_f)
 				usage();
 			opt_f = 1;
 			break;
-		case 'g':
+		GETOPT_OPT("-g"):
 			if (opt_g)
 				usage();
 			opt_g = 1;
 			break;
-		case 'j':
+		GETOPT_OPT("-j"):
 			if (opt_j)
 				usage();
 			opt_j = 1;
 			break;
-		case 'k':
+		GETOPT_OPTARG("-k"):
 			if (opt_k)
 				usage();
 			opt_k = optarg;
 			break;
-		case 'o':
-			if (opt_o != 0.0)
+		GETOPT_OPTARG("-o"):
+			if (opt_o_set)
 				usage();
-			if ((opt_o = strtod(optarg, NULL)) == 0.0) {
+			opt_o_set = 1;
+			if (PARSENUM(&opt_o, optarg, 0, INFINITY)) {
 				warn0("Invalid option: -o %s", optarg);
 				exit(1);
 			}
 			break;
-		case 't':
+		GETOPT_OPTARG("-t"):
 			if (opt_t)
 				usage();
 			opt_t = optarg;
 			break;
-		default:
+		GETOPT_OPT("-v"):
+			fprintf(stderr, "spipe 1.6.0\n");
+			exit(0);
+		GETOPT_MISSING_ARG:
+			warn0("Missing argument to %s\n", ch);
+			usage();
+		GETOPT_DEFAULT:
+			warn0("illegal option -- %s\n", ch);
 			usage();
 		}
 	}
+	argc -= optind;
+	argv += optind;
 
 	/* We should have processed all the arguments. */
-	if (argc != optind)
+	if (argc != 0)
 		usage();
+	(void)argv; /* argv is not used beyond this point. */
 
 	/* Set defaults. */
 	if (opt_o == 0.0)
@@ -123,17 +135,17 @@ main(int argc, char * argv[])
 	/* Resolve target address. */
 	if ((sas_t = sock_resolve(opt_t)) == NULL) {
 		warnp("Error resolving socket address: %s", opt_t);
-		exit(1);
+		goto err0;
 	}
 	if (sas_t[0] == NULL) {
 		warn0("No addresses found for %s", opt_t);
-		exit(1);
+		goto err1;
 	}
 
 	/* Load the keying data. */
 	if ((K = proto_crypt_secret(opt_k)) == NULL) {
 		warnp("Error reading shared secret");
-		exit(1);
+		goto err1;
 	}
 
 	/*
@@ -146,33 +158,50 @@ main(int argc, char * argv[])
 	 */
 	if (socketpair(AF_UNIX, SOCK_STREAM, 0, s)) {
 		warnp("socketpair");
-		exit(1);
+		goto err2;
 	}
 
 	/* Set up a connection. */
-	if (proto_conn_create(s[1], sas_t, 0, opt_f, opt_g, opt_j, K, opt_o,
-	    callback_conndied, NULL)) {
+	if ((conn_cookie = proto_conn_create(s[1], sas_t, 0, opt_f, opt_g,
+	    opt_j, K, opt_o, callback_conndied, &conndone)) == NULL) {
 		warnp("Could not set up connection");
-		exit(1);
+		goto err2;
 	}
 
 	/* Push bits from stdin into the socket. */
-	if (pushbits(STDIN_FILENO, s[0]) || pushbits(s[0], STDOUT_FILENO)) {
+	if (pushbits(STDIN_FILENO, s[0])) {
 		warnp("Could not push bits");
+		goto err3;
+	}
+
+	/* Push bits from the socket to stdout. */
+	if (pushbits(s[0], STDOUT_FILENO)) {
+		warnp("Could not push bits");
+		goto err3;
+	}
+
+	/* Loop until we're done with the connection. */
+	if (events_spin(&conndone)) {
+		warnp("Error running event loop");
 		exit(1);
 	}
 
-	/* Loop until we die. */
-	do {
-		if (events_run()) {
-			warnp("Error running event loop");
-			exit(1);
-		}
-	} while (1);
+	/* Clean up. */
+	events_shutdown();
+	free(K);
 
-	/* NOTREACHED */
-	/*
-	 * If we could reach this point, we would free memory, close sockets,
-	 * and otherwise clean up here.
-	 */
+	/* Success! */
+	exit(0);
+
+err3:
+	proto_conn_drop(conn_cookie);
+	sas_t = NULL;
+	events_shutdown();
+err2:
+	free(K);
+err1:
+	sock_addr_freelist(sas_t);
+err0:
+	/* Failure! */
+	exit(1);
 }
